// This class implements some basic functionality to
// lease event registrations.  
// This class can be extended and used for nearly all
// sorts of event registrations .

package jiniexamples.chapter8;

import net.jini.core.lease.Lease;
import net.jini.core.lease.LeaseMapException;
import net.jini.core.lease.UnknownLeaseException;
import com.sun.jini.lease.landlord.Landlord;
import com.sun.jini.lease.landlord.LandlordLease;
import net.jini.core.event.EventRegistration;
import net.jini.core.event.RemoteEventListener;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.RemoteException;
import java.rmi.MarshalledObject;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;


public class BasicRegistrationLandlord 
                                    extends UnicastRemoteObject
                                    implements Landlord {
    		
    // A simple leasing policy...10-minute leases.
    protected static final int DEFAULT_MAX_LEASE 
                                               = 1000 * 60 * 10;
    protected static long cookiecount = 0;
    
    protected int maxLease = DEFAULT_MAX_LEASE;
    
    // Assume that registrations are kept in
    // a list defined and maintained by the users 
    // of the landlord.
    protected List regs;
    
    
    // A factory for making landlord leases
    protected LandlordLease.Factory factory;
    
    public BasicRegistrationLandlord(List regs, 
                             LandlordLease.Factory factory)
        throws RemoteException {
        this.regs = regs;
        this.factory = factory;
    }


    public List getRegs(){
    	return regs;
    }
    
    
    // Change the maximum lease time from the default
    public void setMaxLease(int maxLease) {
        this.maxLease = maxLease;
    }

    // Apply the policy to a requested duration
    // to get an actual expiration time.
    public long getExpiration(long request) {
        if (request > maxLease || request == Lease.ANY)
            return System.currentTimeMillis() + maxLease;
        else
            return System.currentTimeMillis() + request;
    }


// This adds a registration to the list , creates a lease and
// returns a event registration that can be sent back to the 
// event requester. 

     public  EventRegistration addEventReg(Object cookie,
                                    long eventType,
                                    MarshalledObject data,
                                    RemoteEventListener listener,
                                    Object source,
                                    long duration) 
      {

        // Create the lease and registration, and add the
        // registration to  list
        long expiration = getExpiration(duration);
        
        // New registration 
        Registration reg = new Registration(cookie, 
                                            listener,
                                            data, 
                                            expiration);
        // New Lease 
        Lease lease = factory.newLease(cookie, 
                                       this, 
                                       expiration);
                                       
        // Add the resource... 
        regs.add(reg);
        
        // Create and return an event registration
        EventRegistration evtreg = 
            new EventRegistration(eventType, 
                                  source, 
                                  lease, 
                                  0);
        
        return evtreg;
    }



// ===== The LandLord Interface provided to Clients ============
    // Cancel the lease represented by 'cookie'
    public void cancel(Object cookie) 
                                  throws UnknownLeaseException {
        for (int i=0, size=regs.size() ; i<size ; i++) {
            Registration reg = 
		       (Registration) regs.get(i);
            if (reg.cookie.equals(cookie)) {
                reg.cancelled();
                regs.remove(i);
                return;
            }
        }
        throw new UnknownLeaseException(cookie.toString());
    }

    // Cancel a set of leases
    public Map  cancelAll(Object[] cookies) {
        Map exceptionMap = null;
        LeaseMapException lme = null;
        
        for (int i=0 ; i<cookies.length ; i++) {
            try {
                cancel(cookies[i]);
            } catch (UnknownLeaseException ex) {
                if (exceptionMap == null) {
                    exceptionMap = new HashMap();
                }
                exceptionMap.put(cookies[i], ex);
            }
        }
        
        return exceptionMap;
    }

    // Renew the lease specified by 'cookie'
    public long renew(Object cookie, long extension)
        throws UnknownLeaseException {
        
        for (int i=0, size=regs.size() ; i<size ; i++) {
            Registration reg = 
			(Registration) regs.get(i);
            if (reg.getCookie().equals(cookie)) {
                long expiration = 
			getExpiration(extension);
                reg.setExpiration(expiration);
                return expiration - 
			System.currentTimeMillis();
            }
        }
        throw new UnknownLeaseException(cookie.toString());
    }
    // Renew a set of leases.
    public Landlord.RenewResults renewAll(Object[] cookies,
                                          long[] extensions) {
        long[] granted = new long[cookies.length];
        Exception[] denied = null;
        
        for (int i=0 ; i<cookies.length ; i++) {
            try {
                granted[i] = renew(cookies[i], 
				   extensions[i]);
            } catch (Exception ex) {
                if (denied == null) {
                    denied = 
                        new Exception[cookies.length+1];
                }
                denied[i+1] = ex;
            }
        }
            
        Landlord.RenewResults results =
            new Landlord.RenewResults(granted, denied);
        return results;
    }
// ====== The LandLord Interface provided to Clients ==========

}

